/*
  centerpos.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "qgmap.h"
#include "centerpos.h"
#include "pos.h"
#include "layeredbase.h"
#include "mainwindow.h"
#include "inputdialog.h"
#include "loc_phs.h"

#include <qapplication.h>
#include <qclipboard.h>

CenterPos::CenterPos(LayeredBase *parent, const char *name) :
	LayeredWidget(parent, name)
{
	move_event_counter = 0;
}

Pos CenterPos::getPosition()
{
	return center;
}

void CenterPos::setPosition(Pos &pos)
{
	Enter();
	pos.normalize();
	pos.clipY();
	if(center == pos)
		return;

	center = pos;
	emit positionChanged(center);
}

void CenterPos::setZoom(int zoom)
{
	if(center.getZoom() == zoom)
		return;

	if(center.zoomTo(zoom))
		emit positionChanged(center);
}

void CenterPos::scroll(int x, int y)
{
	setPosition(center.offset(x, y));
}

void CenterPos::zoomup(int z)
{
	if(center.zoomup(z))
		emit positionChanged(center);
}

bool CenterPos::wheelEvent( QWheelEvent *e )
{
	Enter();
	zoomup(e->delta()<0?1:-1);
	return true;
}

bool CenterPos::mouseMoveEvent( QMouseEvent *e )
{
	Enter();

	// イベントの間引きによる高速化
	move_event_counter = (move_event_counter+1)%3;
	if(move_event_counter)
		return true;

	QPoint pos = click_pos - e->pos();

	scroll(pos.x(), pos.y());

	click_pos = e->pos();
	return true;
}

bool CenterPos::mousePressEvent( QMouseEvent *e )
{
	Enter();

	if(e->button() != LeftButton)
		return false;

	click_pos = e->pos();
	return true;
}

bool CenterPos::mouseReleaseEvent( QMouseEvent *e )
{
	Enter();

	if(e->button() != LeftButton)
		return false;

	repaint();
	return true;
}

bool CenterPos::keyPressEvent ( QKeyEvent * e )
{
	int shift = (e->state() & ShiftButton)?SHIFT_MUL:1;


	switch(e->key()){
	case Key_Left:
		scroll(-KEY_SCROLL*shift, 0);
		return true;
	case Key_Right:
		scroll(KEY_SCROLL*shift, 0);
		return true;
	case Key_Up:
		scroll(0, -KEY_SCROLL*shift);
		return true;
	case Key_Down:
		scroll(0, KEY_SCROLL*shift);
		return true;
	case Key_I:
		zoomup(-1);
		return true;
	case Key_O:
		zoomup(1);
		return true;
	case Key_J:
		jumpLocation();
		return true;
	case Key_P:
		getPHSLocation();
		return true;

	// copy & paste
	case Key_C:
		if(e->state() & ControlButton)  // Fn+C も Ctrl+C と同じ
			copyPosition();
		break;
	case Key_V:
		if(e->state() & ControlButton)  // Fn+V も Ctrl+V と同じ
			pastePosition();
		break;

	// Save & Load marker pos
	case Key_0:
	case Key_1:
	case Key_2:
	case Key_3:
	case Key_4:
	case Key_5:
	case Key_6:
	case Key_7:
	case Key_8:
	case Key_9:
		if(e->state() & ShiftButton) {
			savePosition(e->key() - Key_0);
		} else {
			loadPosition(e->key() - Key_0);
		}
		return true;

#ifndef QWS	// for PC
	case Key_Exclam:
		savePosition(1);
		return true;
	case Key_QuoteDbl:
		savePosition(2);
		return true;
	case Key_NumberSign:
		savePosition(3);
		return true;
	case Key_Dollar:
		savePosition(4);
		return true;
	case Key_Percent:
		savePosition(5);
		return true;
	case Key_Ampersand:
		savePosition(6);
		return true;
	case Key_Apostrophe:
		savePosition(7);
		return true;
	case Key_ParenLeft:
		savePosition(8);
		return true;
	case Key_ParenRight:
		savePosition(9);
		return true;
	case Key_AsciiTilde:
		savePosition(0);
		return true;
#endif // for PC
	}
	return false;
}

void CenterPos::showEvent()
{
	resize(parentWidget()->size());
}

void CenterPos::savePosition(int i)
{
	QString key;
	QString info_str;

	key = tr("Pos%1").arg(i);
	((MainWindow *)parentWidget())->setConfig(key, center.serialize());

	info_str = QString::fromUtf8("Pos%1に保存").arg(i);
	((MainWindow *)parentWidget())->dispInfo(info_str);
}

void CenterPos::loadPosition(int i)
{
	Enter();
	QString key, str;
	QString info_str;
	Pos tmp;

	key.sprintf("Pos%d", i);
	str = ((MainWindow *)parentWidget())->getConfig(key, tmp.serialize());

	tmp.deserialize(str);

	setPosition(tmp);

	info_str = QString::fromUtf8("Pos%1を読込").arg(i);
	((MainWindow *)parentWidget())->dispInfo(info_str);
}

void CenterPos::jumpLocation()
{
	InputDialog dlg(parentWidget(), tr("Jump Location (Lat,Lon[,zoom])"), TRUE);
	QString str;

	str.sprintf("%f,%f", center.getLat(), center.getLon());
	dlg.setText(str);

	if(dlg.exec() == QDialog::Accepted){
		Pos pos = center;
		if(pos.setPos(dlg.getText())){
			setPosition(pos);
		}
	}
}

void CenterPos::copyPosition()
{
	QClipboard *cb = QApplication::clipboard();

	cb->setText(center.getPos());
	((MainWindow *)parentWidget())->dispInfo(
		QString::fromUtf8("現在位置をクリップボードへ保存しました"));
}

void CenterPos::pastePosition()
{
	QClipboard *cb = QApplication::clipboard();
	Pos pos = center;

	if(pos.setPos(cb->text())){
		setPosition(pos);
	}else{
		((MainWindow *)parentWidget())->dispInfo(
			QString::fromUtf8("クリップボードの位置情報が不正です"));
	}
}

void CenterPos::getPHSLocation()
{
	LocPHS phs(parentWidget());
	Pos pos = center;

	if(!phs.getLocation(pos))
		return;

	setPosition(pos);
	((MainWindow *)parentWidget())->dispInfo(
		QString::fromUtf8("PHSから位置情報を取得しました"));
}
