/*
  loc_phs.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "loc_phs.h"
#include "pos.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <qdatetime.h>
#include "progressdialog.h"
#include <qapplication.h>
#include <qmessagebox.h>

#define CARDCTL "/sbin/cardctl"
#define MAXRETRY 10

LocPHS::LocPHS(QWidget *parent)
{
	this->parent = parent;
#ifndef QWS
	dummy_suspend = true;
	dummy_count = 0;
#endif
}

LocPHS::~LocPHS()
{
}

bool LocPHS::getLocation(Pos &pos)
{
	QString lat, lon;
	int i;
	bool ret = false;
	ProgressDialog progress(parent);
	bool state;
	QString errmsg;

	if(!isSerialCard()){
		errmsg = QString::fromUtf8("PHSカードが挿入されていません");
		goto err;
	}

	progress.show();
	progress.setCaption(QString::fromUtf8("Get location from PHS"));
	progress.setMax(MAXRETRY + 2);
	progress.setStep(QString::fromUtf8("カードの有効化"),0);

	state = isSuspended();
	if(state){
		resume();
		if(isSuspended()){
			errmsg = QString::fromUtf8("レジュームに失敗しました");
			goto err;
		}
	}

	for(i = 0; i < MAXRETRY; i++){
		progress.setStep(QString::fromUtf8("位置情報取得中・・・"), i + 1);
		if(getLatLon(lat,lon))
			break;
	}
	if(i < MAXRETRY){
		ret = true;
	}

	progress.setStep(QString::fromUtf8("カードの無効化"), MAXRETRY + 1);
	if(state){
		suspend();
	}

	if(!ret){
		errmsg = QString::fromUtf8("位置情報の取得に失敗しました");
		goto err;
	}

	progress.hide();

	if(ret)
		pos.setPos(lat+QString(",")+lon, false);

	return ret;
  err:
	progress.hide();
	QMessageBox::critical(parent, QString::fromUtf8("Error"), errmsg);
	
	return false;
}

bool LocPHS::getLatLon(QString &lat, QString &lon)
{
#ifdef QWS
	static const char *command = 
		"su -c \"pppd ttyS3 connect 'chat -s -v -t 2 ABORT ERROR \\\"\\\" " \
		"AT@LBC1 OK AT@LBC? OK; false;'\"";
	FILE *fp;

	QTime t;

	fp = popen(command, "r");
	if(fp == NULL)
		return false;

	lat = "";
	lon = "";

	t.start();
	while(!feof(fp)){
		char buf[256];
		fgets(buf, sizeof(buf)-1, fp);
		if(strncmp(buf, "Connect", 7) == 0){
			break;
		}
		if(buf[0] == 'N'){
			lat = cleanString(buf + 1);
		}
		if(buf[0] == 'E'){
			lon = cleanString(buf + 1);
			break;
		}

		buf[255]='\0';
	}
	pclose(fp);

	if(lat.length() == 0 || lon.length() == 0)
		return false;

	return true;
#else
	sleep(2);
	dummy_count++;
	if(dummy_count > 5){
		lat = "134.345";
		lon = "35.123";
		return true;
	}
	return false;
#endif
}

bool LocPHS::isSerialCard()
{
	int ret;
#ifdef QWS
	ret = system(CARDCTL" ident 0 | grep -q function.*\\(serial\\)");
#else
	sleep(1);
	ret = 0;
#endif
	return !ret;
}

bool LocPHS::isSuspended()
{
	int ret;
#ifdef QWS
	ret = system(CARDCTL" status 0 | grep -q suspended");
#else
	sleep(1);
	ret = !dummy_suspend;
#endif
	return !ret;
}

void LocPHS::suspend()
{
#ifdef QWS
	system("su -c \""CARDCTL" suspend 0\"");
#else
	sleep(1);
	dummy_suspend = true;
#endif
}

void LocPHS::resume()
{
#ifdef QWS
	system("su -c \""CARDCTL" resume 0\"");
#else
	sleep(1);
	dummy_suspend = false;
	dummy_count = 0;
#endif
}

QString LocPHS::cleanString(const char *p)
{
	QString str = "";
	char c;
	while((c = *p++)){
		if(!isdigit(c) && c != ':')
			break;

		if(c == ':')
			c = '.';

		str.append(c);
	}
	return str;
}
