/*
  map.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "pos.h"
#include "map.h"
#include <qpixmap.h>
#include <qimage.h>
#include <qstring.h>
#include <qpixmapcache.h>
#include <qwmatrix.h>
#include <stdio.h>
#include <qdatetime.h>

Map::Map(int _max_down) :
	img_cache(15,15)
{
	max_down = _max_down;

	// 4MB Cache
	// 256x256x32bpp = 256kB/Picture
	// 4MB/256kB = 16 Picture
	cache.setCacheLimit(4096);

	img_cache.setAutoDelete(TRUE);
}

Map::~Map()
{
	img_cache.clear();
}

QString Map::getPath(Pos &pos)
{
  unsigned int x,y;
  QString strX, strY;
  QString path;
  int i;

  x = pos.getX() / 256;
  y = pos.getY() / 256;

  path.sprintf(base_path+"/%02d", pos.getZoom());
  strX.sprintf("%010d", x);
  strY.sprintf("%010d", y);

  for(i=0;i<10;i++){
    path += '/';
    path += strX.mid(i, 1);
    path += strY.mid(i, 1);
  }

  path += ".png";

  return path;
}

int Map::loadPixmap(Pos &pos)
{
	Pos org_pos = pos.offset(IMAGE_SIZE/2, IMAGE_SIZE/2);
	Pos work_pos;
	QString org_path, path;
	int i=0;
	int max_len = IMAGE_SIZE<<(19-pos.getZoom());

	QTime all;
	all.start();

	org_pos.normalize();
	work_pos = org_pos;

	if(work_pos.getY() < 0 || work_pos.getY() >= max_len){
		pix.resize(256, 256);
		pix.fill(Qt::lightGray);
		return 0;
	}

	org_path = getPath(org_pos);

	// find cache
	if(cache.find(org_path, pix)){
		return 0;
	}

	QImage *img_p;
	img_p = img_cache.find(org_path);
	if(img_p){
		pix.convertFromImage(*img_p);
		return 0;
	}

	// load picture
	if(pix.load(org_path)){
		cache.insert(org_path, pix);
		return 0;
	}

	while( i++ < max_down ){

		if(!work_pos.zoomup(1)){
			// noimage
			break;
		}

		path = getPath(work_pos);

		QImage *work_p;

		// load picture
		work_p = img_cache.find(path);
		if( !work_p ){
			work_p = new QImage();
			if(!work_p->load(path))
				continue;
			img_cache.insert(path, work_p);
		}

		QImage img = zoom(work_p, i, org_pos);
		pix.convertFromImage(img);

		cache.insert(org_path, pix);

		return 0;

	}

	// find no image from cache;
	if(cache.find("noimage", pix)){
		return 0;
	}

	if(pix.load(NOIMG_FILE)){
		cache.insert("noimage", pix);
		return 0;
	}
	return -1;
}

QPixmap &Map::getPix()
{
  return pix;
}

QImage Map::zoom(QImage *img, int z, Pos const &pos)
{
	if(img->depth() < 8){
		img->convertDepth(8);
	}

	QImage ret = img->copy();
	int x,y;
	int pos_x, pos_y;
	int offset_x, offset_y;

	unsigned char *src;
	unsigned char *dst;

	int zoom = 1 << z;

	pos_x = (pos.getX() / IMAGE_SIZE) & (zoom-1);
	pos_y = (pos.getY() / IMAGE_SIZE) & (zoom-1);

	offset_x = IMAGE_SIZE / zoom * pos_x;
	offset_y = IMAGE_SIZE / zoom * pos_y;

	if(img->depth() != 8){
		printf("depth %d is not supported\n", img->depth());
		return ret;
	}

	for(y=0; y<img->height(); y++){
		dst = ret.scanLine(y);
		src = img->scanLine(offset_y + (y>>z));
		for(x=0;x<img->width();x++){
			*dst++ = src[offset_x + (x>>z)];
		}
	}
	return ret;
}

void Map::setMapPath(QString const &path)
{
	base_path = path;
}
